"""
WhatXtract: WhatsApp data extraction and automation toolkit.

WhatsApp Contacts Extractor and Number Checker via WhatsApp Web
A WhatsApp data extraction and automation toolkit

License: MIT
Version: 0.1.0
Author : Hasan Rasel
Email  : rrss.mahmud@gmail.com
"""

# fmt: off
__title__       = 'whatxtract'
__description__ = 'A WhatsApp data extraction and automation toolkit'
__version__     = '0.1.0'
__author__      = 'Hasan Rasel'
__email__       = 'rrss.mahmud@gmail.com'
__license__     = 'MIT'
__url__         = 'https://github.com/bitbytelab/whatxtract'
__copyright__   = '2025 BitByteLab'
